<!DOCTYPE html>
<html>
<head>
    <meta http-equiv="content-type" content="text/html; charset=utf-8">
</head>
<body>
    <div class="sidebar">
        {# we continue to pass the sections key in case someone subclassed the template and use these #}
        {% include "dynamic_preferences/sections.html" with registry=registry sections=registry.sections %}
    </div>
    <div class="main_content">
        {% block content %}{% endblock %}
    </div>
</body>
</html>
